/**************************************************************************** 
*  Copyright 2007, 2008 C&H Technologies, Inc.  All Rights Reserved.					    * 
*****************************************************************************/
/****************************************************************************
*
*	ma204_ansi.c  
*
*	ANSI-C source module for the MA204 50MHz Pulse Generator.
*
* This file contains high level functions to control a MA204 50MHz Pulse 
*	Generator from C&H Technologies, Inc.  This file is written entirely in 
*	ANSI-C so that is may be compiled in any environment with a supporting 
*	ANIS-C compiler It uses the APIS library to perform instrument 
*	communications.  
*
*	This file is organized such that modifications are easy.  All functions 
*	that might need to be modified are located at the end of this file.  
*	Comments are provided that explain tasks of the function and how it might 
*	be modified.  All other function should not need to be modified.
*
*	C&H Part Number: 11028497
*
*	Revision History
*	-	Original Release Version 4.0 - All previos versions where legacy VXIpnp 
*		drivers.  See the legacy VXIpnp source code for details on previous 
*		versions.
*	-	Minor modification to support the release of CHWIN 1.0.
*	-	Release 4.1 (11-15-07 - David Clark)
*	-	Bug fix: ma204_Init was initializing the wait_timeout to 100e-3 ms instead of
*		100ms.  This bug has very little impact (12-19-07 - David Clark).
*	-	Release 4.2 (2-22-08 - David Clark)
*
*****************************************************************************/
#include <ansi_c.h>
#include "ma204_ansi.h"


/**************************************************************************** 
*  Non Exported Defines
*****************************************************************************/
/*  Driver Revision  */
#define		MA204_DRIVER_REV			"4.2"


/**************************************************************************** 
*  Error Checking Macros
*****************************************************************************/
#ifndef checkParamRange
#define  checkParamRange(value, min, max, param)		if ((value > max) || (value < min)) { \
																											error = MA204_ERROR_PARAMETER1 + param-1;\
																											goto Error; \
																										} else 
#endif																				
																				

#ifndef checkParamPointer
#define  checkParamPointer(pointer, param)		if (pointer == NULL) { \
																							error = MA204_ERROR_PARAMETER1 + param-1;\
																							goto Error; \
																							} else 
#endif																				

																				
#ifndef checkErr
#define checkErr(fCall)      if (error = (fCall), (error = (error < 0) ? error : MA204_SUCCESS)) \
                                 {goto Error;}  else
#endif

#ifndef apisCheckErr
#define apisCheckErr(fCall)      if (error = (fCall), (error = (error != 0) ? error : MA204_SUCCESS)) \
                                 {	handle->apis_error = error;\
																		error = MA204_ERROR_APIS; \
																		goto Error;}  else
#endif

#ifndef apisCheckErrInit
#define apisCheckErrInit(fCall)      if (error = (fCall), (error = (error != 0) ? error : MA204_SUCCESS)) \
                                 {	(*handle)->apis_error = error;\
																		error = MA204_ERROR_APIS; \
																		goto Error;}  else
#endif
																				

/**************************************************************************** 
*  Defined Structures
*****************************************************************************/
typedef struct 
{	UInt16 low;
	UInt16 mid;
	UInt16 high;
}MA204_LARGE_NUMBER;



/**************************************************************************** 
*  Constants
*****************************************************************************/
static const char *errorMessageTable[MA204_SPECIFIC_ERROR_END - MA204_SPECIFIC_ERROR_BASE] = 
						{	"The value passed to the function has been reserved for future use.",
							"Wait for ready timeout expired.",
							"The operation cannot be performed while the module is running.",
							"An invalid parameter was passed to the function in parameter 1",
							"An invalid parameter was passed to the function in parameter 2",
							"An invalid parameter was passed to the function in parameter 3",
							"An invalid parameter was passed to the function in parameter 4",
							"The module returned an invalid ID.",
							"An error occurred in the APIS Platform Support Module.  Use the function ma204_GetApisError() to retrieve the APIS error code.",
							"The operation could not be completed because the system is out of memory."
						};

static const char *warningMessageTable[MA204_SPECIFIC_WARNING_END - MA204_SPECIFIC_WARNING_BASE];
						


/**************************************************************************** 
*  Non Exported Functions
*****************************************************************************/
static Int32 ma204_IDWriteByte(MA204_HANDLE handle, UInt8 value);
static Int32 ma204_IDWriteBit(MA204_HANDLE handle, Bool value);
static Int32 ma204_IDReadByte(MA204_HANDLE handle, UInt8 *value);
static Int32 ma204_IDReadBit(MA204_HANDLE handle, Bool *value);
static Int32 ma204_CreateLargeNumber(Real64 number, MA204_LARGE_NUMBER *large_number);
static Int32 ma204_Wait(MA204_HANDLE handle);



/*****************************************************************************
*	ma204_RevisionQuery
*
*	Returns the version number of this driver.  
******************************************************************************/
Int32 ma204_RevisionQuery (Int8 driver_rev[])
{	Int32 error = MA204_SUCCESS;

/*  Check parameters  */
	checkParamPointer(driver_rev, 2);
	
	sprintf(driver_rev, "%s", MA204_DRIVER_REV);
Error:
	return error;
}



/*****************************************************************************
*	ma204_ErrorMessage
*
*	Returns a text description of the specified status code.  
******************************************************************************/
Int32 ma204_ErrorMessage (Int32 statusCode, Int8 errorMessage[])
{	Int32 error = MA204_SUCCESS;
	
/*  Check parameters  */
	checkParamPointer(errorMessage, 3);

/*  Check driver defined error codes  */
	if((statusCode >= MA204_SPECIFIC_ERROR_BASE) && (statusCode < MA204_SPECIFIC_ERROR_END))
	{	strcpy(errorMessage, errorMessageTable[statusCode - MA204_SPECIFIC_ERROR_BASE]);
	}
	else if((statusCode >= MA204_SPECIFIC_WARNING_BASE) && (statusCode < MA204_SPECIFIC_WARNING_END))
	{	strcpy(errorMessage, warningMessageTable[statusCode - MA204_SPECIFIC_WARNING_BASE]);
	}
	else if (statusCode == 0)
		sprintf(errorMessage, "No Error Occured");
	else
		sprintf(errorMessage, "Unknown Error Code [0x%x]", statusCode);
		
Error:
	return error;
}



/*****************************************************************************
*	ma204_GetIdentWord
*
*	Reads the specified offset from the module's ID space and returns
*	the result.
******************************************************************************/
Int32 ma204_GetIdentWord (MA204_HANDLE handle, UInt16 offset, UInt16 *value)
{	Int32 error = MA204_SUCCESS;
	UInt8 data;

/*  Check parameters  */
	checkParamPointer(value, 3);

	*value = 0;
	
/*  Send the read command to the serial EEPROM  */	
	checkErr(ma204_IDWriteByte(handle, 0x80 | offset));
	
/*  Read upper byte  */	
	checkErr(ma204_IDReadByte(handle, &data));
	*value = data << 8;
	
/*  Read lower byte  */	
	checkErr(ma204_IDReadByte(handle, &data));
	*value |= data;
	
Error:	
	ma204_WriteRegister(handle, MA204_ID_ACCESS_REGISTER, 0);
	return error;
}



/*****************************************************************************
*	ma204_WaitForReady
*	
*	Sets the wait for ready flag in the instrument attributes.  If true
*	every function in this driver will wait for the ready bit to become
*	1 before returning.
******************************************************************************/
Int32 ma204_WaitForReady (MA204_HANDLE handle, Bool wait)
{	Int32 error = MA204_SUCCESS;
	
/*  Check parameters  */
	checkParamRange(wait, 0, 1, 2);
	
	handle->wait_for_ready = wait;

Error:
	return error;
}



/*****************************************************************************
*	ma204_SetWaitTimeout
*	
*	Set the wait timeout attribute for the instrument.  The wait timeout
*	is the number of milliseconds the driver will wait for ready before 
*	returning an error
******************************************************************************/
Int32 ma204_SetWaitTimeout (MA204_HANDLE handle, Real64 timeout_ms)
{	Int32 error = MA204_SUCCESS;
	
	handle->wait_timeout = timeout_ms;

Error:
	return error;
}



/*****************************************************************************
*	ma204_SetPeriod
*	
*	This function will program the ma204's period to the  specified
*	number of seconds
******************************************************************************/
Int32 ma204_SetPeriod (MA204_HANDLE handle, Real64 period)
{	Int32 error = MA204_SUCCESS;
	MA204_LARGE_NUMBER large_number;
	
/*  Check parameters  */
	checkParamRange(period, MA204_MIN_PERIOD, MA204_MAX_PERIOD, 2);

/*  Convert from a floating point number to a 36 bit number w/ 100ps resolution */	
	checkErr(ma204_CreateLargeNumber(period*1.0e10, &large_number));

/*  Write the number to the instrument  */	
	checkErr(ma204_WriteRegister(handle, MA204_PULSE_PERIOD_LOW_REGISTER, large_number.low));
	checkErr(ma204_WriteRegister(handle, MA204_PULSE_PERIOD_MID_REGISTER, large_number.mid));
	checkErr(ma204_WriteRegister(handle, MA204_PULSE_PERIOD_HIGH_REGISTER, large_number.high));
	
/*  Wait for ready  */	
	checkErr(ma204_Wait(handle));
Error:
	return error;
}



/*****************************************************************************
*	ma204_SetPulseWidth
*	
*	This function will program the ma204's pulse width to the  specified
*	number of seconds
******************************************************************************/
Int32 ma204_SetPulseWidth (MA204_HANDLE handle, Real64 pulseWidth)
{	Int32 error = MA204_SUCCESS;
	MA204_LARGE_NUMBER large_number;
	
/*  Check parameters  */
	checkParamRange(pulseWidth, MA204_MIN_PULSE_WIDTH, MA204_MAX_PULSE_WIDTH, 2);

/*  Convert from a floating point number to a 36 bit number w/ 100ps resolution */	
	checkErr(ma204_CreateLargeNumber(pulseWidth*1.0e10, &large_number));

/*  Write the number to the instrument  */	
	checkErr(ma204_WriteRegister(handle, MA204_PULSE_WIDTH_LOW_REGISTER, large_number.low));
	checkErr(ma204_WriteRegister(handle, MA204_PULSE_WIDTH_MID_REGISTER, large_number.mid));
	checkErr(ma204_WriteRegister(handle, MA204_PULSE_WIDTH_HIGH_REGISTER, large_number.high));

/*  Wait for ready  */	
	checkErr(ma204_Wait(handle));
Error:
	return error;
}


/*****************************************************************************
*	ma204_SetDelay
*	
*	This function will program the ma204's delay to the specified
*	number of seconds
******************************************************************************/
Int32 ma204_SetDelay (MA204_HANDLE handle, Real64 delay)
{	Int32 error = MA204_SUCCESS;
	MA204_LARGE_NUMBER large_number;
	
/*  Check parameters  */
	checkParamRange(delay, MA204_MIN_DELAY, MA204_MAX_DELAY, 2);
	
/*  Convert from a floating point number to a 36 bit number w/ 100ps resolution */	
	checkErr(ma204_CreateLargeNumber(delay*1.0e10, &large_number));

/*  Write the number to the instrument  */	
	checkErr(ma204_WriteRegister(handle, MA204_PULSE_DELAY_LOW_REGISTER, large_number.low));
	checkErr(ma204_WriteRegister(handle, MA204_PULSE_DELAY_MID_REGISTER, large_number.mid));
	checkErr(ma204_WriteRegister(handle, MA204_PULSE_DELAY_HIGH_REGISTER, large_number.high));

/*  Wait for ready  */	
	checkErr(ma204_Wait(handle));
Error:
	return error;
}


/*****************************************************************************
*	ma204_SetPulsePairDelay
*	
*	This function will program the ma204's pulse pair delay to the specified
*	number of seconds
******************************************************************************/
Int32 ma204_SetPulsePairDelay (MA204_HANDLE handle, Real64 delay)
{	Int32 error = MA204_SUCCESS;
	MA204_LARGE_NUMBER large_number;
	
/*  Check parameters  */
	checkParamRange(delay, MA204_MIN_DELAY, MA204_MAX_DELAY, 2);
	
/*  Convert from a floating point number to a 36 bit number w/ 100ps resolution */	
	checkErr(ma204_CreateLargeNumber(delay*1.0e10, &large_number));

/*  Write the number to the instrument  */	
	checkErr(ma204_WriteRegister(handle, MA204_PULSE_PAIR_DELAY_LOW_REGISTER, large_number.low));
	checkErr(ma204_WriteRegister(handle, MA204_PULSE_PAIR_DELAY_MID_REGISTER, large_number.mid));
	checkErr(ma204_WriteRegister(handle, MA204_PULSE_PAIR_DELAY_HIGH_REGISTER, large_number.high));

/*  Wait for ready  */	
	checkErr(ma204_Wait(handle));
Error:
	return error;
}



/*****************************************************************************
*	ma204_SetAmplitude
*	
*	This function will program the ma204's high and low voltage levels to 
*	the specified voltages
******************************************************************************/
Int32 ma204_SetAmplitude (MA204_HANDLE handle, Real64 highLevel, Real64 lowLevel)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;
	UInt16 revision;
	
/*  Check parameters  */
	checkParamRange(highLevel,MA204_MIN_VOLTAGE, MA204_MAX_VOLTAGE, 2); 
	checkParamRange(lowLevel,MA204_MIN_VOLTAGE, MA204_MAX_VOLTAGE, 3); 

/*  Write Low Level  */
/*  Convert to a 16-bit number with 2.2 mV resolution and a -2V offset  */
	data = (UInt16)((lowLevel + 2.0)/2.2e-3);
	checkErr(ma204_WriteRegister(handle, MA204_OUTPUT_AMPLITUDE_LOW_REGISTER, data));
	
/*  Write High Level  */
/*  Convert to a 16-bit number with 2.2 mV resolution and a -2V offset  */
	data = (UInt16)((highLevel + 2.0)/2.2e-3);
	checkErr(ma204_WriteRegister(handle, MA204_OUTPUT_AMPLITUDE_HIGH_REGISTER, data));
	
/*  Wait for ready  */	
	checkErr(ma204_Wait(handle));
Error:
	return error;
}



/*****************************************************************************
*	ma204_SetRunMode
*	
*	Sets the run mode to the specified value.  The run mode determines
*	if the ma204 runs continuously, once, a specified number of times, or if 
*	it follows the input trigger.
******************************************************************************/
Int32 ma204_SetRunMode (MA204_HANDLE handle, Int32 runMode)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;
	Int32 running = MA204_FALSE;
	
/*  Check parameters  */	
	checkParamRange(runMode, 0, 3, 2);
	
/*  If running then return error  */	
	checkErr(ma204_ReadRegister(handle, MA204_CONTROL_STATUS_REGISTER, &data));
	if(data & MA204_RUN_BIT)   
	{	error = MA204_ERROR_RUNNING;
		goto Error;
	}
	
/*  Set the run bits to the specified value  */	
	data &= ~MA204_RUN_MODE_BITS;
	data |= runMode << 1;

	checkErr(ma204_WriteRegister(handle, MA204_CONTROL_STATUS_REGISTER, data));

/*  Wait for ready  */	
	checkErr(ma204_Wait(handle));
Error:
	return error;
}


/*****************************************************************************
*	ma204_SetPulsePair
*	
*	Sets or clears the pulse pair bit.  This bit determines whether the ma204
*	outputs a single pulse or a pule pair each period
******************************************************************************/
Int32 ma204_SetPulsePair (MA204_HANDLE handle, Bool pulsePair)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;

/*  Check parameters  */	
	checkParamRange(pulsePair, 0, 1, 2);

/*  Set or clear the pulse pair bit  */
	checkErr(ma204_ReadRegister(handle, MA204_CONTROL_STATUS_REGISTER, &data));
	if(pulsePair == MA204_TRUE)
		data |= MA204_PULSE_PAIR_BIT;
	else
		data &= ~MA204_PULSE_PAIR_BIT;

/*  Write the data to the instrument  */
	checkErr(ma204_WriteRegister(handle, MA204_CONTROL_STATUS_REGISTER, data));

/*  Wait for ready  */	
	checkErr(ma204_Wait(handle));
Error:
	return error;
}


/*****************************************************************************
*	ma204_SetBurstCount
*	
*	Writes the specified value to the burst count register.  If the ma204
*	is set to burst mode than this is the number of pulses or pulse pairs
*	that will be output.
******************************************************************************/
Int32 ma204_SetBurstCount (MA204_HANDLE handle, UInt32 count)
{	Int32 error = MA204_SUCCESS;

/*  Write the count to the instrument  */
	checkErr(ma204_WriteRegister(handle, MA204_BURST_COUNT_LOW_REGISTER, (count & 0x0000FFFF)));
	checkErr(ma204_WriteRegister(handle, MA204_BURST_COUNT_HIGH_REGISTER, ((count>>16) & 0x0000FFFF)));
	
/*  Wait for ready  */	
	checkErr(ma204_Wait(handle));
Error:
	return error;
}


/*****************************************************************************
*	ma204_SetPulsePolarity
*	
*	Sets or clears the pulse polarity bit.  This bit determines the logical
*	polarity of the ma204, active high or active low.
******************************************************************************/
Int32 ma204_SetPulsePolarity (MA204_HANDLE handle, Bool polarity)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;
	
/*  Check parameters  */	
	checkParamRange(polarity, 0, 1, 2);

/*  Set or clear the pulse polarity bit  */
	checkErr(ma204_ReadRegister(handle, MA204_CONTROL_STATUS_REGISTER, &data));
	if(polarity == MA204_TRUE)
		data |= MA204_PULSE_POLARITY_BIT;
	else
		data &= ~MA204_PULSE_POLARITY_BIT;

/*  Write the data to the instrument  */
	checkErr(ma204_WriteRegister(handle, MA204_CONTROL_STATUS_REGISTER, data));

/*  Wait for ready  */	
	checkErr(ma204_Wait(handle));
	
Error:
	return error;
}


/*****************************************************************************
*	ma204_SetPulseOutputImpedance 
*	
* Sets or clears the pulse output impedance bit.  This bit will
*	select either 50 Ohms or low (2-5 ohms) output impedance
******************************************************************************/
Int32 ma204_SetPulseOutputImpedance (MA204_HANDLE handle, Bool impedance)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;
	
/*  Check parameters  */	
	checkParamRange(impedance, 0, 1, 2);

/*  Set or clear the impedance bit  */
	checkErr(ma204_ReadRegister(handle, MA204_CONTROL_STATUS_REGISTER, &data));
	if(impedance == MA204_TRUE)
		data |= MA204_OUTPUT_IMPEDANCE_BIT;
	else
		data &= ~MA204_OUTPUT_IMPEDANCE_BIT;

/*  Write the data to the instrument  */
	checkErr(ma204_WriteRegister(handle, MA204_CONTROL_STATUS_REGISTER, data));

/*  Wait for ready  */	
	checkErr(ma204_Wait(handle));
	
Error:
	return error;
}



/*****************************************************************************
*	ma204_SetPulseOutputSlewRate 
*	
* Sets the slew rate of the pulse output signal.
*****************************************************************************/
Int32 ma204_SetPulseOutputSlewRate (MA204_HANDLE handle, Real64 slewRate)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;
	
/*  Check Parameters  */	
	checkParamRange(slewRate, MA204_MIN_SLEW_RATE, MA204_MAX_SLEW_RATE, 2);

/*  Calculate slew rate value and write to the register  */	
	data = (UInt16)((slewRate - MA204_MIN_SLEW_RATE)/MA204_SLEW_RATE_RESOLUTION);
	checkErr(ma204_WriteRegister(handle, MA204_SLEW_RATE_REGISTER, data));
	
/*  Wait for ready  */	
	checkErr(ma204_Wait(handle));
Error:
	return error;
}



/*****************************************************************************
*	ma204_SetCalibrationValue
*	
* Sets the calibration register to the specified value
*****************************************************************************/
Int32 ma204_SetCalibrationValue (MA204_HANDLE handle, UInt16 calibrationValue)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;
	
/*  Check parameters  */	
	checkParamRange(calibrationValue, 0, 0x0FFF, 2);

/*  Read the current value of the calibration register  */
	checkErr(ma204_ReadRegister(handle, MA204_CALIBRATION_REGISTER, &data));
	
/*  Clear the calibration reset bit  */	
	data &= ~MA204_CALIBRATION_RESET_BIT;
	checkErr(ma204_WriteRegister(handle, MA204_CALIBRATION_REGISTER, data));
	
/*  Write the calibration value to the calibration register  */	
	data = calibrationValue;
	checkErr(ma204_WriteRegister(handle, MA204_CALIBRATION_REGISTER, data));

Error:
	return error;
}



/*****************************************************************************
*	ma204_ConfigureMTriggers
*	
*	Configures the MA204 to use M-Module Triggers A and B as inputs,
* output of pulse out, or output of snyc out
******************************************************************************/
Int32 ma204_ConfigureMTriggers (MA204_HANDLE handle, Int32 m_trigger_a, Int32 m_trigger_b)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;

/*  Check Parameters  */
	checkParamRange(m_trigger_a, 0, 3, 2);
	checkParamRange(m_trigger_b, 0, 3, 3);
	if((m_trigger_a == 3) || (m_trigger_b == 3))
	{	error = MA204_ERROR_RESERVED_VALUE;
		goto Error;
	}
	
/*  Read the trigger/gate control register  */	
	checkErr(ma204_ReadRegister(handle, MA204_TRIGGER_GATE_CONTROL_REGISTER, &data));
	
/*  Set the trigger select bits to the desired value  */	
	data &= ~(MA204_MTRGSELB_BITS | MA204_MTRGSELA_BITS);
	data |= (m_trigger_a << 4) | (m_trigger_b << 6);
	
/*  Write the value to the trigger/gate control register  */	
	checkErr(ma204_WriteRegister(handle, MA204_TRIGGER_GATE_CONTROL_REGISTER, data));

/*  Wait for ready  */	
	checkErr(ma204_Wait(handle));
Error:
	return error;
}



/*****************************************************************************
*	ma204_ConfigureGateSource
*	
*	Configures the source for the gate signal and sets the gate mode
*	to either synchronous or asynchronous
******************************************************************************/
Int32 ma204_ConfigureGateSource (MA204_HANDLE handle, Bool enable, Bool mode, Int32 select)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;

/*  Check parameters  */
	checkParamRange(enable, 0, 1, 2);
	checkParamRange(mode, 0, 1, 3);
	checkParamRange(select, 0, 15, 4);
	if((select == 3) || (select == 6) || (select == 7) || (select == 8) || (select == 11) || (select == 14) || (select == 15))
	{ error = MA204_ERROR_RESERVED_VALUE;
		goto Error;
	}
	

/*  Get current value of the trigger/gate control register  */
	checkErr(ma204_ReadRegister(handle, MA204_TRIGGER_GATE_CONTROL_REGISTER, &data));

/*  Set the synchronous gate mode bits  */
	if(mode == MA204_SYNCHRONOUS_GATE_MODE)
		data |= MA204_SYNCHRONOUS_GATING_BIT;
	else 		
		data &= ~MA204_SYNCHRONOUS_GATING_BIT;

/*  Set the gate select bits  */
	data &= ~MA204_GATE_SELECT_BITS;	
	data |= select << 8;
	
/*  Set the gate mode (enable/disable) bit  */	
	if(enable == MA204_TRUE)
		data |= MA204_GATE_MODE_BIT;
	else
		data &= ~MA204_GATE_MODE_BIT;

/*  Write the new value to the trigger/gate control register  */	
	checkErr(ma204_WriteRegister(handle, MA204_TRIGGER_GATE_CONTROL_REGISTER, data));

/*  Wait for ready  */	
	checkErr(ma204_Wait(handle));
Error:
	return error;
}



/*****************************************************************************
*	ma204_ConfigureTriggerSource
*	
*	Configure the source of the trigger signal	
******************************************************************************/
Int32 ma204_ConfigureTriggerSource (MA204_HANDLE handle, Int32 select)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;

/*  Check parameters 	*/
	checkParamRange(select, 0, 15, 2);
	if((select == 3) || (select == 6) || (select == 7) || (select == 8) || (select == 11) || (select == 14) || (select == 15))
	{ error = MA204_ERROR_RESERVED_VALUE;
		goto Error;
	}
	
/*  Read the current value of the trigger/gate control register  */
	checkErr(ma204_ReadRegister(handle, MA204_TRIGGER_GATE_CONTROL_REGISTER, &data));
	
/*  Set the trigger select bits and write the new value to the register  */	
	data &= ~MA204_TRIGGER_SELECT_BITS;
	data |= select;
	checkErr(ma204_WriteRegister(handle, MA204_TRIGGER_GATE_CONTROL_REGISTER, data));
	
/*  Wait for ready  */	
	checkErr(ma204_Wait(handle));
Error:
	return error;
}



/*****************************************************************************
*	ma204_ConfigureSyncOut 
*	
*	Enables or disables sync out and set the sync out polarity to active
* high or active low
******************************************************************************/
Int32 ma204_ConfigureSyncOut (MA204_HANDLE handle, Bool enable, Bool polarity)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;
	
/*  Check Parameters  */	
	checkParamRange(enable, 0, 1, 2);
	checkParamRange(polarity, 0, 1, 3);
	
/*  Read the current value of the control/status register  */	
	checkErr(ma204_ReadRegister(handle, MA204_CONTROL_STATUS_REGISTER, &data));
	
/*  Set the sync out enable bit  */	
		data &= ~(MA204_SYNC_OUT_ENABLE_BIT | MA204_SYNC_OUT_POLARITY_BIT);
		data |= (enable << 12) | (polarity << 13);

/*  Write the new value to the register  */		
	checkErr(ma204_WriteRegister(handle, MA204_CONTROL_STATUS_REGISTER, data));
	
/*  Wait for ready  */	
	checkErr(ma204_Wait(handle));
Error:
	return error;
}



/*****************************************************************************
*	ma204_ConfigureExternalReference 
*	
*	Enable or disable the use of the external reference and set
* the source of the external reference signal
******************************************************************************/
Int32 ma204_ConfigureExternalReference (MA204_HANDLE handle, Bool enable, Int32 select)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;

/*  Check Parameters  */	
	checkParamRange(enable, 0, 1, 2);
	checkParamRange(select, 0, 3, 3);
	
/*  Read the current value of the control/status register  */	
	checkErr(ma204_ReadRegister(handle, MA204_CONTROL_STATUS_REGISTER, &data));
	
/*  Set the external reference enable and select bits  */	
	data &= ~(MA204_REFERENCE_SELECT_BITS | MA204_REFERENCE_ENABLE_BIT);
	data |= (select<<8) | (enable << 11);

/*  Write the new value to the register  */	
	checkErr(ma204_WriteRegister(handle, MA204_CONTROL_STATUS_REGISTER, data));
	
/*  Wait for ready  */	
	checkErr(ma204_Wait(handle));
Error:
	return error;
}



/*****************************************************************************
*	ma204_ConfigurePulseOutputCharacteristics
*	
*	Configures the slew rate, the output impedance, and the polarity
*	of the output pulse.
******************************************************************************/
Int32 ma204_ConfigurePulseOutputCharacteristics (MA204_HANDLE handle, Real64 slewRate, Bool impedance, Bool polarity)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;
	
/*  Check Parameters  */	
	checkParamRange(slewRate, MA204_MIN_SLEW_RATE, MA204_MAX_SLEW_RATE, 2);
	checkParamRange(impedance, 0, 1, 3);
	checkParamRange(polarity, 0, 1, 4);

/*  Calculate Slew Rate Program Value and Write to Slew Rate Register  */	
	data = (UInt16)((slewRate - MA204_MIN_SLEW_RATE)/MA204_SLEW_RATE_RESOLUTION);
	checkErr(ma204_WriteRegister(handle, MA204_SLEW_RATE_REGISTER, data));
	
/*  Read the current value of the control/status register  */	
	checkErr(ma204_ReadRegister(handle, MA204_CONTROL_STATUS_REGISTER, &data));
	
/*  Set the impedance and polarity bits  */
	data &= ~(MA204_OUTPUT_IMPEDANCE_BIT | MA204_PULSE_POLARITY_BIT);
	data |= (impedance << 6) | (polarity << 5);
	
/*  Write the new value to the register  */	
	checkErr(ma204_WriteRegister(handle, MA204_CONTROL_STATUS_REGISTER, data));

/*  Wait for ready  */	
	checkErr(ma204_Wait(handle));
Error:
	return error;
}



/*****************************************************************************
*	ma204_ConfigIntConditions
*	
*	Configures the conditions on which the ma204 will generate an
*	interrupt.  The conditions that can be configured are End of Burst
*	and Ready Bit.
******************************************************************************/
Int32 ma204_ConfigIntConditions (MA204_HANDLE handle, Bool endOfBurst, Bool readyBit)
{	Int32 error = MA204_SUCCESS;
	UInt16 value;

/*  Check Parameters  */
	checkParamRange(endOfBurst, 0, 1, 2);
	checkParamRange(readyBit, 0, 1, 3);

/*  Enable/Disable Interrupt End of Burst Interrupt  */
	checkErr(ma204_ReadRegister(handle, MA204_INTERRUPT_CONTROL_REGISTER, &value));
	value &= ~(MA204_END_OF_BURST_INTERRUPT_ENABLE_BIT | MA204_READY_BIT_INTERRUPT_ENABLE_BIT);	
	
	value |= endOfBurst | (readyBit << 1);
	checkErr(ma204_WriteRegister(handle, MA204_INTERRUPT_CONTROL_REGISTER, value));
	
Error:
	return error;
}



/*****************************************************************************
*	ma204_Run
*	
*	Sets the run bit of the ma204
******************************************************************************/
Int32 ma204_Run (MA204_HANDLE handle)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;
	
/*  Read the current value of the Control/Status register  */
	checkErr(ma204_ReadRegister(handle, MA204_CONTROL_STATUS_REGISTER, &data));
	
/*  Set the run bit  */	
	data |= MA204_RUN_BIT;
	checkErr(ma204_WriteRegister(handle, MA204_CONTROL_STATUS_REGISTER, data));
	
/*  Wait for ready  */	
	checkErr(ma204_Wait(handle));	
	
Error:
	return error;
}



/*****************************************************************************
*	ma204_Stop
*	
*	Clears the run bit of the ma204
******************************************************************************/
Int32 ma204_Stop (MA204_HANDLE handle)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;
	
/*  Read the current value of the Control/Status register  */
	checkErr(ma204_ReadRegister(handle, MA204_CONTROL_STATUS_REGISTER, &data));

/*  Clear the run bit  */	
	data &= ~MA204_RUN_BIT;
	checkErr(ma204_WriteRegister(handle, MA204_CONTROL_STATUS_REGISTER, data));

/*  Wait for ready  */	
	checkErr(ma204_Wait(handle));	
	
Error:
	return error;
}



/*****************************************************************************
*	ma204_OutputEnable
*	
*	Enable/Disable the output relay
******************************************************************************/
Int32 ma204_OutputEnable (MA204_HANDLE handle, Bool enabled)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;
	
/*  Check Parameters  */	
	checkParamRange(enabled, 0, 1, 2);

/*  Read the current value of the Control/Status register  */
	checkErr(ma204_ReadRegister(handle, MA204_CONTROL_STATUS_REGISTER, &data));

/*  Set/Clear the output enable bit  */	
	if(enabled == MA204_TRUE)
		data |= MA204_OUTPUT_ENABLED_BIT;
	else
		data &= ~MA204_OUTPUT_ENABLED_BIT;
	checkErr(ma204_WriteRegister(handle, MA204_CONTROL_STATUS_REGISTER, data));
	
Error:
	return error;
}



/*****************************************************************************
*	ma204_ExternalReferenceStatus
*	
*	Returns the value of the reference detected bit and the oscillator
*	locked bit.
******************************************************************************/
Int32 ma204_ExternalReferenceStatus (MA204_HANDLE handle, Bool *referenceDetected, Bool *oscillatorLocked)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;
	
/*  Check Parameters  */
	checkParamPointer(referenceDetected, 2);
	checkParamPointer(oscillatorLocked, 3);

/*  Read the Control/Status Register  */	
	checkErr(ma204_ReadRegister(handle, MA204_CONTROL_STATUS_REGISTER, &data));
	
/*  Set the status parameters  */	
	if(data & MA204_REFERENCE_DETECED_BIT)
		*referenceDetected = MA204_TRUE;
	else
		*referenceDetected = MA204_FALSE;
		
	if(data & MA204_OSCILLATOR_LOCKED_BIT)
		*oscillatorLocked = MA204_TRUE;
	else
		*oscillatorLocked = MA204_FALSE;
		
Error:
	return error;
}



/*****************************************************************************
*	ma204_Ready
*	
*	Returns the value of the ready bit
******************************************************************************/
Int32 ma204_Ready (MA204_HANDLE handle, Bool *ready)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;
	
/*  Check Paremeters  */	
	checkParamPointer(ready, 2);
	
/*  Read the value of the ready bit  */	
	checkErr(ma204_ReadRegister(handle, MA204_CONTROL_STATUS_REGISTER, &data));
	if(data & MA204_READY_BIT)
		*ready = MA204_TRUE;
	else
		*ready = MA204_FALSE;
		
Error:
	return error;
}



/*****************************************************************************
*	ma204_EndOfBurst
*	
*	Returns the value of the End of Burst flag
******************************************************************************/
Int32 ma204_EndOfBurst (MA204_HANDLE handle, Bool *endOfBurst)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;
	
/*  Check Paremeters  */	
	checkParamPointer(endOfBurst, 2);
	
/*  Read the value of the end of burst flag bit  */	
	checkErr(ma204_ReadRegister(handle, MA204_INTERRUPT_CONTROL_REGISTER, &data));
	if(data & MA204_END_OF_BURST_FLAG_BIT)
		*endOfBurst = MA204_TRUE;
	else
		*endOfBurst = MA204_FALSE;
		
Error:
	return error;
}



/*****************************************************************************
*	ma204_ClearEndOfBurst
*	
*	Clears the End of Burst Flag by writting a 1 to the bit
******************************************************************************/
Int32 ma204_ClearEndOfBurst (MA204_HANDLE handle)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;
	
/*  Read the current value if the interrupt control register  */	
	checkErr(ma204_ReadRegister(handle, MA204_INTERRUPT_CONTROL_REGISTER, &data));
	
/*  Write a 1 to the end of burst flag bit  */	
	data |= MA204_END_OF_BURST_FLAG_BIT;
	checkErr(ma204_WriteRegister(handle, MA204_INTERRUPT_CONTROL_REGISTER, data));
		
Error:
	return error;
}


/*****************************************************************************
*	ma204_CalibrationReset
*	
*	Reset the calibration value to its default
******************************************************************************/
Int32 ma204_CalibrationReset (MA204_HANDLE handle)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;
	
/*  Set the calibration reset bit  */	
	checkErr(ma204_WriteRegister(handle, MA204_CALIBRATION_REGISTER, 0x8FFF));

Error:
	return error;
}



/*****************************************************************************
*	ma204_ClearInterrupts
*	
*	Clears the End of Burst interrupt
******************************************************************************/
Int32 ma204_ClearInterrupts (MA204_HANDLE handle, Bool endOfBurst, Bool readyBit)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;
	
/*  Check Parameters  */
	checkParamRange(endOfBurst, 0, 1, 2);
	checkParamRange(readyBit, 0, 1, 3);

/*  Read the current value of the interrupt control register  */	
	checkErr(ma204_ReadRegister(handle, MA204_INTERRUPT_CONTROL_REGISTER, &data));

	data &= ~(MA204_END_OF_BURST_FLAG_BIT | MA204_READY_BIT_FLAG_BIT);

/*  Clear end of burst interrupt  */
	if(endOfBurst == MA204_TRUE)
		data |= MA204_END_OF_BURST_FLAG_BIT;
		
/*  Clear ready bit interrupt  */
	if(readyBit == MA204_TRUE)
		data |= MA204_READY_BIT_FLAG_BIT;

/*  Write the new value to the interrupt control register  */
	checkErr(ma204_WriteRegister(handle, MA204_INTERRUPT_CONTROL_REGISTER, data));
		
Error:
	return error;
}



/*****************************************************************************
*	ma204_GetInterruptPending
*	
*	Returns the status of the end of burst interrupt and the ready bit interrupt
******************************************************************************/
Int32 ma204_GetInterruptPending (MA204_HANDLE handle, Bool *endOfBurst, Bool *readyBit)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;

/*  Check Paremeters  */	
	checkParamPointer(endOfBurst, 2);
	checkParamPointer(readyBit, 3);
	
/*  Read the value of the interrupt control register  */	
	checkErr(ma204_ReadRegister(handle, MA204_INTERRUPT_CONTROL_REGISTER, &data));
	
/*  Check the end of burst falg bit  */	
	if(data & MA204_END_OF_BURST_FLAG_BIT)
		*endOfBurst = MA204_TRUE;
	else
		*endOfBurst = MA204_FALSE;
		
/*  Check the ready bit flag bit  */		
	if(data & MA204_READY_BIT_FLAG_BIT)
		*readyBit = MA204_TRUE;
	else
		*readyBit = MA204_FALSE;

Error:
	return error;
}



/*****************************************************************************
*	ma204_CreateContinuousPulse 
*	
*	Configures the ma204 to generate a continuous pulse cycle of the 
* specified period, pulse width, amplitude, etc.
******************************************************************************/
Int32 ma204_CreateContinuousPulse (MA204_HANDLE handle, Real64 pulseWidth, Real64 period, Real64 delay, 
                                        Real64 highLevel, Real64 lowLevel)
{	Int32 error = MA204_SUCCESS;
	Bool wait_for_ready;
	
/*  Check parameters  */
	checkParamRange(pulseWidth, MA204_MIN_PULSE_WIDTH, MA204_MAX_PULSE_WIDTH, 2);
	checkParamRange(period, MA204_MIN_PERIOD, MA204_MAX_PERIOD, 3);
	checkParamRange(delay, MA204_MIN_DELAY, MA204_MAX_DELAY, 4);
	checkParamRange(highLevel,MA204_MIN_VOLTAGE, MA204_MAX_VOLTAGE, 5); 
	checkParamRange(lowLevel,MA204_MIN_VOLTAGE, MA204_MAX_VOLTAGE, 6); 

/*  get the initial value of the wait for ready flag  */
	wait_for_ready = handle->wait_for_ready;
	
/*  Program the ma204  */
	checkErr(ma204_Stop (handle));
	checkErr(ma204_WaitForReady(handle, 0));
	checkErr(ma204_SetPeriod(handle, period));
	checkErr(ma204_SetPulseWidth(handle, pulseWidth));
	checkErr(ma204_SetDelay(handle, delay));
	checkErr(ma204_SetAmplitude(handle, highLevel, lowLevel));
	checkErr(ma204_SetPulsePair(handle, MA204_FALSE));
	checkErr(ma204_SetRunMode(handle, 1));
	checkErr(ma204_ConfigureSyncOut(handle, 1, 0));
	checkErr(ma204_OutputEnable(handle, 1));
	checkErr(ma204_WaitForReady(handle, 1));
	checkErr(ma204_Wait(handle));
	checkErr(ma204_Run (handle));
	
/*  Reset the wait for ready flag to its initial value  */
	if(!wait_for_ready)
		checkErr(ma204_WaitForReady(handle, 0));
Error:
	return error;
}



/*****************************************************************************
*	ma204_CreateContinuousPulsePair 
*	
*	Configures the ma204 to generate a continuous pulse pair cycle of the 
* specified period, pulse width, amplitude, etc.
******************************************************************************/
Int32 ma204_CreateContinuousPulsePair (MA204_HANDLE handle,
                                            Real64 pulseWidth, Real64 period,
                                            Real64 delay, Real64 highLevel,
                                            Real64 lowLevel,
                                            Real64 pulsePairDelay)
{	Int32 error = MA204_SUCCESS;
	Bool wait_for_ready;
	
/*  Check parameters  */
	checkParamRange(pulseWidth, MA204_MIN_PULSE_WIDTH, MA204_MAX_PULSE_WIDTH, 2);
	checkParamRange(period, MA204_MIN_PERIOD, MA204_MAX_PERIOD, 3);
	checkParamRange(delay, MA204_MIN_DELAY, MA204_MAX_DELAY, 4);
	checkParamRange(highLevel,MA204_MIN_VOLTAGE, MA204_MAX_VOLTAGE, 5); 
	checkParamRange(lowLevel,MA204_MIN_VOLTAGE, MA204_MAX_VOLTAGE, 6); 
	checkParamRange(pulsePairDelay, MA204_MIN_DELAY, MA204_MAX_DELAY, 7);

/*  get the initial value of the wait for ready flag  */
	wait_for_ready = handle->wait_for_ready;

/*  Program the ma204  */
	checkErr(ma204_Stop (handle));
	checkErr(ma204_WaitForReady(handle, 0));
	checkErr(ma204_SetPeriod(handle, period));
	checkErr(ma204_SetPulseWidth(handle, pulseWidth));
	checkErr(ma204_SetDelay(handle, delay));
	checkErr(ma204_SetAmplitude(handle, highLevel, lowLevel));
	checkErr(ma204_SetPulsePair(handle, MA204_TRUE));
	checkErr(ma204_SetRunMode(handle, 1));
	checkErr(ma204_SetPulsePairDelay (handle, pulsePairDelay));
	checkErr(ma204_ConfigureSyncOut(handle, 1, 0));
	checkErr(ma204_OutputEnable (handle, 1));
	checkErr(ma204_WaitForReady(handle, 1));
	checkErr(ma204_Wait(handle));
	checkErr(ma204_Run (handle));
	
/*  Reset the wait for ready flag to its initial value  */
	if(!wait_for_ready)
		checkErr(ma204_WaitForReady(handle, 0));
Error:
	return error;
}



/*****************************************************************************
*	ma204_CreateSinglePulse
*	
*	Configures the ma204 to generate a single pulse cycle of the 
* specified period, pulse width, amplitude, etc.
******************************************************************************/
Int32 ma204_CreateSinglePulse (MA204_HANDLE handle, Real64 pulseWidth, Real64 period, Real64 delay,
                                    Real64 highLevel, Real64 lowLevel)
{	Int32 error = MA204_SUCCESS;
	Bool wait_for_ready;
	
/*  Check parameters  */
	checkParamRange(pulseWidth, MA204_MIN_PULSE_WIDTH, MA204_MAX_PULSE_WIDTH, 2);
	checkParamRange(period, MA204_MIN_PERIOD, MA204_MAX_PERIOD, 3);
	checkParamRange(delay, MA204_MIN_DELAY, MA204_MAX_DELAY, 4);
	checkParamRange(highLevel,MA204_MIN_VOLTAGE, MA204_MAX_VOLTAGE, 5); 
	checkParamRange(lowLevel,MA204_MIN_VOLTAGE, MA204_MAX_VOLTAGE, 6); 

/*  get the initial value of the wait for ready flag  */
	wait_for_ready = handle->wait_for_ready;

/*  Program the ma204  */
	checkErr(ma204_WaitForReady(handle, 0));
	checkErr(ma204_Stop (handle));
	checkErr(ma204_SetPeriod(handle, period));
	checkErr(ma204_SetPulseWidth(handle, pulseWidth));
	checkErr(ma204_SetDelay(handle, delay));
	checkErr(ma204_SetAmplitude(handle, highLevel, lowLevel));
	checkErr(ma204_SetPulsePair(handle, MA204_FALSE));
	checkErr(ma204_SetRunMode (handle, 0));
	checkErr(ma204_ConfigureSyncOut(handle, 1, 0));
	checkErr(ma204_OutputEnable (handle, 1));
	checkErr(ma204_WaitForReady(handle, 1));
	checkErr(ma204_Wait(handle));
	checkErr(ma204_Run (handle));

/*  Reset the wait for ready flag to its initial value  */
	if(!wait_for_ready)
		checkErr(ma204_WaitForReady(handle, 0));
	
Error:
	return error;
}



/*****************************************************************************
*	ma204_CreateSinglePulsePair
*	
*	Configures the ma204 to generate a single pulse pair cycle of the 
* specified period, pulse width, amplitude, etc.
******************************************************************************/
Int32 ma204_CreateSinglePulsePair (MA204_HANDLE handle, Real64 pulseWidth, Real64 period, Real64 delay,
                                    Real64 highLevel, Real64 lowLevel, Real64 pulsePairDelay)
{	Int32 error = MA204_SUCCESS;
	Bool wait_for_ready;
	
/*  Check parameters  */
	checkParamRange(pulseWidth, MA204_MIN_PULSE_WIDTH, MA204_MAX_PULSE_WIDTH, 2);
	checkParamRange(period, MA204_MIN_PERIOD, MA204_MAX_PERIOD, 3);
	checkParamRange(delay, MA204_MIN_DELAY, MA204_MAX_DELAY, 4);
	checkParamRange(highLevel,MA204_MIN_VOLTAGE, MA204_MAX_VOLTAGE, 5); 
	checkParamRange(lowLevel,MA204_MIN_VOLTAGE, MA204_MAX_VOLTAGE, 6); 
	checkParamRange(pulsePairDelay, MA204_MIN_DELAY, MA204_MAX_DELAY, 7);

/*  get the initial value of the wait for ready flag  */
	wait_for_ready = handle->wait_for_ready;

/*  Program the ma204  */
	checkErr(ma204_WaitForReady(handle, 0));
	checkErr(ma204_Stop (handle));
	checkErr(ma204_SetPeriod(handle, period));
	checkErr(ma204_SetPulseWidth(handle, pulseWidth));
	checkErr(ma204_SetDelay(handle, delay));
	checkErr(ma204_SetAmplitude(handle, highLevel, lowLevel));
	checkErr(ma204_SetPulsePair(handle, MA204_TRUE));
	checkErr(ma204_SetRunMode (handle, 0));
	checkErr(ma204_SetPulsePairDelay (handle, pulsePairDelay));
	checkErr(ma204_ConfigureSyncOut(handle, 1, 0));
	checkErr(ma204_OutputEnable (handle, 1));
	checkErr(ma204_WaitForReady(handle, 1));
	checkErr(ma204_Wait(handle));
	checkErr(ma204_Run (handle));

/*  Reset the wait for ready flag to its initial value  */
	if(!wait_for_ready)
		checkErr(ma204_WaitForReady(handle, 0));
	
Error:
	return error;
}



/*****************************************************************************
*	ma204_CreatePulseBurst
*	
*	Configures the ma204 to generate a pulse burst of the 
* specified count, period, pulse width, amplitude, etc.
******************************************************************************/
Int32 ma204_CreatePulseBurst (MA204_HANDLE handle, Real64 pulseWidth, Real64 period, Real64 delay,
                                   Real64 highLevel, Real64 lowLevel, UInt32 burstCount)
{	Int32 error = MA204_SUCCESS;
	Bool wait_for_ready;
	
/*  Check parameters  */
	checkParamRange(pulseWidth, MA204_MIN_PULSE_WIDTH, MA204_MAX_PULSE_WIDTH, 2);
	checkParamRange(period, MA204_MIN_PERIOD, MA204_MAX_PERIOD, 3);
	checkParamRange(delay, MA204_MIN_DELAY, MA204_MAX_DELAY, 4);
	checkParamRange(highLevel,MA204_MIN_VOLTAGE, MA204_MAX_VOLTAGE, 5); 
	checkParamRange(lowLevel,MA204_MIN_VOLTAGE, MA204_MAX_VOLTAGE, 6); 

/*  get the initial value of the wait for ready flag  */
	wait_for_ready = handle->wait_for_ready;

/*  Program ma204  */
	checkErr(ma204_WaitForReady(handle, 0));
	checkErr(ma204_Stop(handle));
	checkErr(ma204_SetPeriod(handle, period));
	checkErr(ma204_SetPulseWidth(handle, pulseWidth));
	checkErr(ma204_SetDelay(handle, delay));
	checkErr(ma204_SetAmplitude(handle, highLevel, lowLevel));
	checkErr(ma204_SetPulsePair(handle, MA204_FALSE));
	checkErr(ma204_SetRunMode (handle, 2));
	checkErr(ma204_SetBurstCount (handle, burstCount));
	checkErr(ma204_ConfigureSyncOut(handle, 1, 0));
	checkErr(ma204_OutputEnable (handle, 1));
	checkErr(ma204_WaitForReady(handle, 1));
	checkErr(ma204_Wait(handle));
	checkErr(ma204_Run (handle));
	
/*  Reset the wait for ready flag to its initial value  */
	if(!wait_for_ready)
		checkErr(ma204_WaitForReady(handle, 0));
Error:
	return error;
}



/*****************************************************************************
*	ma204_CreatePulsePairBurst
*	
*	Configures the ma204 to generate a pulse pair burst of the 
* specified count, period, pulse width, amplitude, etc.
******************************************************************************/
Int32 ma204_CreatePulsePairBurst (MA204_HANDLE handle, Real64 pulseWidth, Real64 period, Real64 delay,
                                   Real64 highLevel, Real64 lowLevel, Real64 pulsePairDelay, UInt32 burstCount)
{	Int32 error = MA204_SUCCESS;
	Bool wait_for_ready;
	
/*  Check parameters  */
	checkParamRange(pulseWidth, MA204_MIN_PULSE_WIDTH, MA204_MAX_PULSE_WIDTH, 2);
	checkParamRange(period, MA204_MIN_PERIOD, MA204_MAX_PERIOD, 3);
	checkParamRange(delay, MA204_MIN_DELAY, MA204_MAX_DELAY, 4);
	checkParamRange(highLevel,MA204_MIN_VOLTAGE, MA204_MAX_VOLTAGE, 5); 
	checkParamRange(lowLevel,MA204_MIN_VOLTAGE, MA204_MAX_VOLTAGE, 6); 
	checkParamRange(pulsePairDelay, MA204_MIN_DELAY, MA204_MAX_DELAY, 7);

/*  get the initial value of the wait for ready flag  */
	wait_for_ready = handle->wait_for_ready;

/*  Program ma204  */
	checkErr(ma204_WaitForReady(handle, 0));
	checkErr(ma204_Stop(handle));
	checkErr(ma204_SetPeriod(handle, period));
	checkErr(ma204_SetPulseWidth(handle, pulseWidth));
	checkErr(ma204_SetDelay(handle, delay));
	checkErr(ma204_SetAmplitude(handle, highLevel, lowLevel));
	checkErr(ma204_SetPulsePair(handle, MA204_TRUE));
	checkErr(ma204_SetRunMode (handle, 2));
	checkErr(ma204_SetPulsePairDelay (handle, pulsePairDelay));
	checkErr(ma204_SetBurstCount (handle, burstCount));
	checkErr(ma204_ConfigureSyncOut(handle, 1, 0));
	checkErr(ma204_OutputEnable (handle, 1));
	checkErr(ma204_WaitForReady(handle, 1));
	checkErr(ma204_Wait(handle));
	checkErr(ma204_Run (handle));
	
/*  Reset the wait for ready flag to its initial value  */
	if(!wait_for_ready)
		checkErr(ma204_WaitForReady(handle, 0));
Error:
	return error;
}



/*****************************************************************************
*	ma204_CreateSquareWave
*	
*	Creates a square wave of the specified period, amplitude, and duty cycle
******************************************************************************/
Int32 ma204_CreateSquareWave (MA204_HANDLE handle, Real64 period, UInt16 dutyCycle, Real64 highLevel, Real64 lowLevel)
{	Int32 error = MA204_SUCCESS;
	Real64 pulse_width;
	Bool wait_for_ready;
	
/*  Check parameters  */
	checkParamRange(period, MA204_MIN_PERIOD, MA204_MAX_PERIOD, 2);
	checkParamRange(dutyCycle, 10, 90, 3); 
	checkParamRange(highLevel,MA204_MIN_VOLTAGE, MA204_MAX_VOLTAGE, 4); 
	checkParamRange(lowLevel,MA204_MIN_VOLTAGE, MA204_MAX_VOLTAGE, 5); 

/*  get the initial value of the wait for ready flag  */
	wait_for_ready = handle->wait_for_ready;
	
/*  Program the ma204  */	
	pulse_width = period * ((Real64)dutyCycle/100.0);
	checkErr(ma204_WaitForReady(handle, 0));
	checkErr(ma204_Stop(handle));
	checkErr(ma204_SetPeriod(handle, period));
	checkErr(ma204_SetPulseWidth(handle, pulse_width));
	checkErr(ma204_SetDelay(handle, 0));
	checkErr(ma204_SetAmplitude(handle, highLevel, lowLevel));
	checkErr(ma204_SetPulsePair(handle, MA204_FALSE));
	checkErr(ma204_SetRunMode (handle, 1));
	checkErr(ma204_ConfigureSyncOut(handle, 1, 0));
	checkErr(ma204_OutputEnable (handle, 1));
	checkErr(ma204_WaitForReady(handle, 1));
	checkErr(ma204_Run (handle));
	
/*  Reset the wait for ready flag to its initial value  */
	if(!wait_for_ready)
		checkErr(ma204_WaitForReady(handle, 0));
Error:
	return error;
}



/*****************************************************************************
*	ma204_FollowTriggerInput
*	
*	Configure the ma204 to follow the trigger input
******************************************************************************/
Int32 ma204_FollowTriggerInput (MA204_HANDLE handle, Int32 trigger, Real64 highLevel, Real64 lowLevel)
{	Int32 error = MA204_SUCCESS;

/*  Check parameters  */
	checkParamRange(trigger, 0, 15, 2);
	if((trigger == 3) || (trigger == 6) || (trigger == 7) || (trigger == 8) || (trigger == 11) || (trigger == 14) || (trigger == 15))
	{ error = MA204_ERROR_RESERVED_VALUE;
		goto Error;
	}
	checkParamRange(highLevel,MA204_MIN_VOLTAGE, MA204_MAX_VOLTAGE, 3); 
	checkParamRange(lowLevel,MA204_MIN_VOLTAGE, MA204_MAX_VOLTAGE, 4); 

/*  Program the ma204  */	
	checkErr(ma204_Stop(handle));
	checkErr(ma204_ConfigureTriggerSource (handle, trigger));
	checkErr(ma204_SetAmplitude(handle, highLevel, lowLevel));
	checkErr(ma204_SetRunMode (handle, 3));
	checkErr(ma204_Run (handle));
    
Error:
	return error;
}



/*****************************************************************************
*	ma204_IDWriteByte
*
*	Write a command byte to the ID PROM of the instrument.
*	The instruments ID PROM is serially accessed
*****************************************************************************/
static Int32 ma204_IDWriteByte(MA204_HANDLE handle, UInt8 value)
{	Int32 error = MA204_SUCCESS;
	Int32 i;
	
/*  Initialize  */	
	checkErr(ma204_WriteRegister(handle, MA204_ID_ACCESS_REGISTER, 0));
	checkErr(ma204_WriteRegister(handle, MA204_ID_ACCESS_REGISTER, 0x4));
	
/*  Write start bit  */	
	checkErr(ma204_IDWriteBit(handle, 1));
	for(i=0; i<=7; i++)
	{
	/*  Write data bits  */		
		checkErr(ma204_IDWriteBit(handle, ((value &0x80)>>7)));
		value = value << 1;
	}
Error:	
	return error;
}



/*****************************************************************************
*	ma204_IDWriteBit
*
*	Write a single bit to the ID PROM of the instrument.
*	The instruments ID PROM is serially accessed
*****************************************************************************/
static Int32 ma204_IDWriteBit(MA204_HANDLE handle, Bool value)
{	Int32 error = MA204_SUCCESS;

/*  data with clock low  */
	checkErr(ma204_WriteRegister(handle, MA204_ID_ACCESS_REGISTER, (0x4 | (value & 0x1))));	

/*  data with clock high  */	 
	checkErr(ma204_WriteRegister(handle, MA204_ID_ACCESS_REGISTER, (0x6 | (value & 0x1))));	
Error:	
	return error;
}



/*****************************************************************************
*	ma204_IDReadByte
*
*	Reads a byte from the ID PROM of the instrument.
*	The instruments ID PROM is serially accessed
*****************************************************************************/
static Int32 ma204_IDReadByte(MA204_HANDLE handle, UInt8 *value)
{	Int32 error = MA204_SUCCESS;
	Int32 i;
	Bool data;
	
	*value = 0;
	for(i=7; i>=0; i--)
	{ 
	/*  read data 1 bit at a time  */	
		checkErr(ma204_IDReadBit(handle, &data));
		*value |= ((data & 0x1) << i);
	}
	
Error:	
	return error;
}



/*****************************************************************************
*	ma204_IDReadBit
*
*	Reads a single bit from the ID PROM of the instrument.
*	The instruments ID PROM is serially accessed
*****************************************************************************/
static Int32 ma204_IDReadBit(MA204_HANDLE handle, Bool *value)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;

/*  clock low  */ 	
	checkErr(ma204_WriteRegister(handle, MA204_ID_ACCESS_REGISTER, 0x4));	
	
/*  clock high  */	
	checkErr(ma204_WriteRegister(handle, MA204_ID_ACCESS_REGISTER, 0x6));	
	
/*  Read data  */	
	checkErr(ma204_ReadRegister(handle, MA204_ID_ACCESS_REGISTER, &data));
	*value = (Bool)data;

Error:	
	return error;
}



/*****************************************************************************
*	ma204_CreateLargeNumber
*	
*	Translates a 64 bit real number into a 36 bit binary number
******************************************************************************/
static Int32 ma204_CreateLargeNumber(Real64 number, MA204_LARGE_NUMBER *large_number)
{	Int32 error = MA204_SUCCESS;
	Int32 count;
	Real64 divided_number;
	UInt32 divided_int;
	
	
/*  Determine number of bits above bit 31.  
		We must divide instead of shift since we are dealing with a floating point
		number.
*/	
	count = 0;
	divided_number = number;
	while(divided_number > (Real64)0xFFFFFFFFl)
	{	divided_number /= 2.0;
		count++;
	}
	divided_int = (UInt32)divided_number;
	
/*  Set the high mid and low numbers  */	
	large_number->high = divided_int >> (32 - count);
	large_number->mid = (divided_int >> (16 - count)) & 0x0000FFFF;
	large_number->low = ((divided_int << count) & 0x0000FFFF) | (UInt32)fmod(number, pow(2.0, count));
	
	if(count == 0)
		large_number->high = 0;  // if we shift 32 we do not get 0
	
Error:
	return error;
}


/*****************************************************************************
*	ma204_EnableInterrupts 
*
*	Configures the ma204 to produce interrupts.  This function does not
*	configure the conditions on which interrupts occur nor does it configure
*	the carrier to pass interrupts on to the host controller.
******************************************************************************/
Int32 ma204_EnableInterrupts (MA204_HANDLE handle, Bool interruptEnable)
{	Int32 error = MA204_SUCCESS;
	UInt16 value = 0;
	
/*  Check parameters  */
	checkParamRange(interruptEnable, 0, 1, 2);
	
/*  Configure the MA204 to use ROAK interrupts */	
	checkErr(ma204_ReadRegister(handle, MA204_INTERRUPT_CONTROL_REGISTER, &value));
	if(interruptEnable == MA204_TRUE)
		value |= (MA204_MASTER_INTERRUPT_ENABLE_BIT | MA204_INTERRUPT_TYPE_BIT);
	else
		value &= ~(MA204_MASTER_INTERRUPT_ENABLE_BIT | MA204_INTERRUPT_TYPE_BIT);
	checkErr(ma204_WriteRegister(handle, MA204_INTERRUPT_CONTROL_REGISTER, value));
	
Error:
	return error;
}


/*****************************************************************************
*	ma204_ReEnableInterrupts
*
*	Set the master interrupt enable bit to 1.  This bit is cleared on an 
*	interrupt and must be reset for future interrupts to occur.
******************************************************************************/
Int32 ma204_ReEnableInterrupts (MA204_HANDLE handle)
{	Int32 error = MA204_SUCCESS;
	UInt16 value;
	
/*  Set the master interrupt enable bit to 1  */
	checkErr(ma204_ReadRegister(handle, MA204_INTERRUPT_CONTROL_REGISTER, &value));
	value |= MA204_MASTER_INTERRUPT_ENABLE_BIT | MA204_INTERRUPT_TYPE_BIT;
	checkErr(ma204_WriteRegister(handle, MA204_INTERRUPT_CONTROL_REGISTER, value));

Error:
	return error;
}


/*****************************************************************************
*	Modify the functions below this point to use the MA204 driver in an 
*	environment not supported by an APIS platform support module. All functions
*	above this point should not have to be modified.
*
*	In general, any call to an APIS function (apis_func()) will need to be
*	modified to communicate with the instrument or operating system being used.
*
******************************************************************************/
																																 
/*****************************************************************************
*	ma204_Wait
*	
*	Waits for either the ready bit to go high or for a timeout to occur.	
******************************************************************************/
static Int32 ma204_Wait(MA204_HANDLE handle)
{	Int32 error = MA204_SUCCESS;
	UInt16 data;
	Real64 start_time = 0;
	
	if(handle->wait_for_ready == MA204_TRUE)
	{	start_time = apis_tick();
		ma204_ReadRegister(handle, MA204_CONTROL_STATUS_REGISTER, &data);
		while(((data & MA204_READY_BIT) == 0) && ((apis_tick() - start_time) < handle->wait_timeout))
		{	ma204_ReadRegister(handle, MA204_CONTROL_STATUS_REGISTER, &data);
		}
		if((data & MA204_READY_BIT) == 0)
			error =	MA204_ERROR_WAIT_TIMEOUT_EXPIRED;
	}

Error:
	return error;
}


/*****************************************************************************
*	ma204_Init
*
******************************************************************************/
Int32 ma204_Init (Int32 instr, Bool IDQuery, Bool reset, MA204_HANDLE *handle)
{	Int32 error = MA204_SUCCESS;
	Int32 warning = MA204_SUCCESS;
  UInt16 value;

/*  Check parameters  */
	checkParamRange(IDQuery, 0, 1, 2);
	checkParamRange(reset, 0, 1, 3);
	checkParamPointer(handle, 4);

/*  Allocate the data structure  */
	(*handle) = 0;
	(*handle) = malloc(sizeof(MA204_INSTR));
	if((*handle) == NULL)
	{	error = MA204_ERROR_OUT_OF_MEMORY;
		goto Error;
	}

	(*handle)->apis_handle = 0;
	apisCheckErrInit(apis_open((APIS_PATH)instr, &(*handle)->apis_handle));

	
/*  Initialize Attributes  */
  (*handle)->wait_for_ready = MA204_TRUE;
  (*handle)->wait_timeout = 100;


/*  Perform ID Query  */
	if(IDQuery == MA204_TRUE)
	{	checkErr(ma204_GetIdentWord (*handle, 17, &value));
		value &= 0x0FFF;
		if (value != (UInt16)MA204_VXI_MANF_ID) 
		{	error = MA204_ERROR_FAIL_ID_QUERY;
 			goto Error;
	  }

		checkErr(ma204_GetIdentWord (*handle, 18, &value));
		value &= 0x0FFF;
		if (value != (UInt16)MA204_VXI_MODEL_CODE) 
		{	error = MA204_ERROR_FAIL_ID_QUERY;
			goto Error;
		}
	}

/* Reset Instrument */
  if (reset == MA204_TRUE) 
  {	checkErr(ma204_Reset (*handle));
  }


Error:
/*  Do not free handle here.  The Calling function must call ma204_Close upon receiving an error  */	
		
	if((error == MA204_SUCCESS) && (warning != MA204_SUCCESS))		// if no errors then return the warnings		
		error = warning;

	return error;
}


/*****************************************************************************
*	ma204_Close
*
*	Close the session to the instrument.
******************************************************************************/
Int32 ma204_Close (MA204_HANDLE handle)
{	Int32 error = MA204_SUCCESS;

	if(handle->apis_handle != NULL)
		apisCheckErr(apis_close(handle->apis_handle));
	free(handle);
Error:
	return error;
}


/*****************************************************************************
*	ma204_ReadRegister
*
*	Read the specified offset from the module's I/O space and returns
*	the result.
******************************************************************************/
Int32 ma204_ReadRegister (MA204_HANDLE handle, UInt16 offset, UInt16 *value)
{	Int32 error = MA204_SUCCESS;
	
/*  Check parameters  */
	checkParamPointer(value, 3);
	
	apisCheckErr(apis_read(handle->apis_handle, 2, offset, value));
Error:
	return error;
}


/*****************************************************************************
*	ma204_WriteRegister
*
*	Writes to the specified offset in the module's I/O space
******************************************************************************/
Int32 ma204_WriteRegister (MA204_HANDLE handle, UInt16 offset, UInt16 value)
{	Int32 error = MA204_SUCCESS;

	apisCheckErr(apis_write(handle->apis_handle, 2, offset, value));
Error:
	return error;
}


/*****************************************************************************
*	ma204_reset
*	
*	Reset the instrument to it's default state
******************************************************************************/
Int32 ma204_Reset (MA204_HANDLE handle)
{	
	Int32 error = MA204_SUCCESS;
	
	apisCheckErr(apis_reset(handle->apis_handle));
	
Error:
	return error;}


/*****************************************************************************
*	ma204_InstallIrq
*	
*	Installs the interrupts handler to be called when an interrupt occurs
*	and configures the carrier to pass interrupts onto the system.
******************************************************************************/
Int32 ma204_InstallIrq (MA204_HANDLE handle, void *interruptHandler, Int32 vector, Int32 level, Int32 mode, void *var_ptr)
{	Int32 error = MA204_SUCCESS;

/*  Check parameters  */
	checkParamPointer(interruptHandler, 2);

	apisCheckErr(apis_irqinstall(handle->apis_handle, interruptHandler, vector, level, mode, var_ptr)); 

Error:
	return error;
}


/*****************************************************************************
*	ma204_UninstallIrq
*
*	Un-installs the interrupt handler for the instrument and disables
*	interrupt handling for the instrument
******************************************************************************/
Int32 ma204_UninstallIrq (MA204_HANDLE handle)
{	Int32 error = MA204_SUCCESS;

	apisCheckErr(apis_irqremove(handle->apis_handle));

Error:
	return error;
}


/*****************************************************************************
*	ma204_GetApisError
*
*	Returns the saved apis error code
******************************************************************************/
Int32 ma204_GetApisError (MA204_HANDLE handle, UInt32 *apis_error)
{	Int32 error = MA204_SUCCESS;
	*apis_error = handle->apis_error;
	return error;
}
